/****************************************************************************
** Meta object code from reading C++ file 'customwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../source/common/customwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'customwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19CustomSuggestButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomSuggestButton::qt_create_metaobjectdata<qt_meta_tag_ZN19CustomSuggestButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomSuggestButton"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomSuggestButton, qt_meta_tag_ZN19CustomSuggestButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomSuggestButton::staticMetaObject = { {
    QMetaObject::SuperData::link<DSuggestButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19CustomSuggestButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19CustomSuggestButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19CustomSuggestButtonE_t>.metaTypes,
    nullptr
} };

void CustomSuggestButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomSuggestButton *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomSuggestButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomSuggestButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19CustomSuggestButtonE_t>.strings))
        return static_cast<void*>(this);
    return DSuggestButton::qt_metacast(_clname);
}

int CustomSuggestButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DSuggestButton::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14CustomComboboxE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomCombobox::qt_create_metaobjectdata<qt_meta_tag_ZN14CustomComboboxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomCombobox"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomCombobox, qt_meta_tag_ZN14CustomComboboxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomCombobox::staticMetaObject = { {
    QMetaObject::SuperData::link<DComboBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CustomComboboxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CustomComboboxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14CustomComboboxE_t>.metaTypes,
    nullptr
} };

void CustomCombobox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomCombobox *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomCombobox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomCombobox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CustomComboboxE_t>.strings))
        return static_cast<void*>(this);
    return DComboBox::qt_metacast(_clname);
}

int CustomCombobox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DComboBox::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN16CustomPushButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomPushButton::qt_create_metaobjectdata<qt_meta_tag_ZN16CustomPushButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomPushButton"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomPushButton, qt_meta_tag_ZN16CustomPushButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomPushButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QPushButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CustomPushButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CustomPushButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16CustomPushButtonE_t>.metaTypes,
    nullptr
} };

void CustomPushButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomPushButton *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomPushButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomPushButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16CustomPushButtonE_t>.strings))
        return static_cast<void*>(this);
    return QPushButton::qt_metacast(_clname);
}

int CustomPushButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPushButton::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN18CustomSwitchButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomSwitchButton::qt_create_metaobjectdata<qt_meta_tag_ZN18CustomSwitchButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomSwitchButton"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomSwitchButton, qt_meta_tag_ZN18CustomSwitchButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomSwitchButton::staticMetaObject = { {
    QMetaObject::SuperData::link<DSwitchButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CustomSwitchButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CustomSwitchButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18CustomSwitchButtonE_t>.metaTypes,
    nullptr
} };

void CustomSwitchButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomSwitchButton *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomSwitchButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomSwitchButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CustomSwitchButtonE_t>.strings))
        return static_cast<void*>(this);
    return DSwitchButton::qt_metacast(_clname);
}

int CustomSwitchButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DSwitchButton::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14CustomCheckBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomCheckBox::qt_create_metaobjectdata<qt_meta_tag_ZN14CustomCheckBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomCheckBox"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomCheckBox, qt_meta_tag_ZN14CustomCheckBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomCheckBox::staticMetaObject = { {
    QMetaObject::SuperData::link<DCheckBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CustomCheckBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CustomCheckBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14CustomCheckBoxE_t>.metaTypes,
    nullptr
} };

void CustomCheckBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomCheckBox *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomCheckBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomCheckBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14CustomCheckBoxE_t>.strings))
        return static_cast<void*>(this);
    return DCheckBox::qt_metacast(_clname);
}

int CustomCheckBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DCheckBox::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN23CustomCommandLinkButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomCommandLinkButton::qt_create_metaobjectdata<qt_meta_tag_ZN23CustomCommandLinkButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomCommandLinkButton"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomCommandLinkButton, qt_meta_tag_ZN23CustomCommandLinkButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomCommandLinkButton::staticMetaObject = { {
    QMetaObject::SuperData::link<DCommandLinkButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23CustomCommandLinkButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23CustomCommandLinkButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN23CustomCommandLinkButtonE_t>.metaTypes,
    nullptr
} };

void CustomCommandLinkButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomCommandLinkButton *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomCommandLinkButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomCommandLinkButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN23CustomCommandLinkButtonE_t>.strings))
        return static_cast<void*>(this);
    return DCommandLinkButton::qt_metacast(_clname);
}

int CustomCommandLinkButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DCommandLinkButton::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN21CustomFloatingMessageE_t {};
} // unnamed namespace

template <> constexpr inline auto CustomFloatingMessage::qt_create_metaobjectdata<qt_meta_tag_ZN21CustomFloatingMessageE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CustomFloatingMessage"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CustomFloatingMessage, qt_meta_tag_ZN21CustomFloatingMessageE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CustomFloatingMessage::staticMetaObject = { {
    QMetaObject::SuperData::link<DFloatingMessage::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21CustomFloatingMessageE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21CustomFloatingMessageE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21CustomFloatingMessageE_t>.metaTypes,
    nullptr
} };

void CustomFloatingMessage::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CustomFloatingMessage *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CustomFloatingMessage::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CustomFloatingMessage::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21CustomFloatingMessageE_t>.strings))
        return static_cast<void*>(this);
    return DFloatingMessage::qt_metacast(_clname);
}

int CustomFloatingMessage::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DFloatingMessage::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
