/****************************************************************************
** Meta object code from reading C++ file 'clipzipplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../clipzipplugin.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'clipzipplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20CliPzipPluginFactoryE_t {};
} // unnamed namespace

template <> constexpr inline auto CliPzipPluginFactory::qt_create_metaobjectdata<qt_meta_tag_ZN20CliPzipPluginFactoryE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CliPzipPluginFactory"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CliPzipPluginFactory, qt_meta_tag_ZN20CliPzipPluginFactoryE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CliPzipPluginFactory::staticMetaObject = { {
    QMetaObject::SuperData::link<KPluginFactory::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20CliPzipPluginFactoryE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20CliPzipPluginFactoryE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20CliPzipPluginFactoryE_t>.metaTypes,
    nullptr
} };

void CliPzipPluginFactory::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CliPzipPluginFactory *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *CliPzipPluginFactory::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CliPzipPluginFactory::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20CliPzipPluginFactoryE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "org.kde.KPluginFactory"))
        return static_cast< KPluginFactory*>(this);
    return KPluginFactory::qt_metacast(_clname);
}

int CliPzipPluginFactory::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KPluginFactory::qt_metacall(_c, _id, _a);
    return _id;
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_CliPzipPluginFactory[] = {
    0xbf, 
    // "IID"
    0x02,  0x76,  'o',  'r',  'g',  '.',  'k',  'd', 
    'e',  '.',  'K',  'P',  'l',  'u',  'g',  'i', 
    'n',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x74,  'C',  'l',  'i',  'P',  'z',  'i', 
    'p',  'P',  'l',  'u',  'g',  'i',  'n',  'F', 
    'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa4,  0x67,  'K',  'P',  'l',  'u',  'g', 
    'i',  'n',  0xa8,  0x6b,  'D',  'e',  's',  'c', 
    'r',  'i',  'p',  't',  'i',  'o',  'n',  0x78, 
    0x34,  'H',  'i',  'g',  'h',  '-',  'p',  'e', 
    'r',  'f',  'o',  'r',  'm',  'a',  'n',  'c', 
    'e',  ' ',  'p',  'a',  'r',  'a',  'l',  'l', 
    'e',  'l',  ' ',  'Z',  'I',  'P',  ' ',  'c', 
    'o',  'm',  'p',  'r',  'e',  's',  's',  'i', 
    'o',  'n',  ' ',  'u',  's',  'i',  'n',  'g', 
    ' ',  'p',  'z',  'i',  'p',  0x72,  'D',  'e', 
    's',  'c',  'r',  'i',  'p',  't',  'i',  'o', 
    'n',  '[',  'z',  'h',  '_',  'C',  'N',  ']', 
    0x78,  0x2c,  uchar('\xe4'), uchar('\xbd'), uchar('\xbf'), uchar('\xe7'), uchar('\x94'), uchar('\xa8'),
    ' ',  'p',  'z',  'i',  'p',  ' ',  uchar('\xe8'), uchar('\xbf'),
    uchar('\x9b'), uchar('\xe8'), uchar('\xa1'), uchar('\x8c'), uchar('\xe9'), uchar('\xab'), uchar('\x98'), uchar('\xe6'),
    uchar('\x80'), uchar('\xa7'), uchar('\xe8'), uchar('\x83'), uchar('\xbd'), uchar('\xe5'), uchar('\xb9'), uchar('\xb6'),
    uchar('\xe8'), uchar('\xa1'), uchar('\x8c'), ' ',  'Z',  'I',  'P',  ' ', 
    uchar('\xe5'), uchar('\x8e'), uchar('\x8b'), uchar('\xe7'), uchar('\xbc'), uchar('\xa9'), 0x62,  'I', 
    'd',  0x71,  'k',  'e',  'r',  'f',  'u',  'f', 
    'f',  'l',  'e',  '_',  'c',  'l',  'i',  'p', 
    'z',  'i',  'p',  0x69,  'M',  'i',  'm',  'e', 
    'T',  'y',  'p',  'e',  's',  0x81,  0x6f,  'a', 
    'p',  'p',  'l',  'i',  'c',  'a',  't',  'i', 
    'o',  'n',  '/',  'z',  'i',  'p',  0x64,  'N', 
    'a',  'm',  'e',  0x6b,  'p',  'z',  'i',  'p', 
    ' ',  'p',  'l',  'u',  'g',  'i',  'n',  0x6b, 
    'N',  'a',  'm',  'e',  '[',  'z',  'h',  '_', 
    'C',  'N',  ']',  0x78,  0x1a,  'p',  'z',  'i', 
    'p',  ' ',  uchar('\xe9'), uchar('\xab'), uchar('\x98'), uchar('\xe6'), uchar('\x80'), uchar('\xa7'),
    uchar('\xe8'), uchar('\x83'), uchar('\xbd'), uchar('\xe5'), uchar('\x8e'), uchar('\x8b'), uchar('\xe7'), uchar('\xbc'),
    uchar('\xa9'), uchar('\xe6'), uchar('\x8f'), uchar('\x92'), uchar('\xe4'), uchar('\xbb'), uchar('\xb6'), 0x6c, 
    'S',  'e',  'r',  'v',  'i',  'c',  'e',  'T', 
    'y',  'p',  'e',  's',  0x81,  0x70,  'K',  'e', 
    'r',  'f',  'u',  'f',  'f',  'l',  'e',  '/', 
    'P',  'l',  'u',  'g',  'i',  'n',  0x67,  'V', 
    'e',  'r',  's',  'i',  'o',  'n',  0x63,  '1', 
    '.',  '0',  0x78,  0x19,  'X',  '-',  'K',  'D', 
    'E',  '-',  'K',  'e',  'r',  'f',  'u',  'f', 
    'f',  'l',  'e',  '-',  'R',  'e',  'a',  'd', 
    'W',  'r',  'i',  't',  'e',  0xf5,  0x6e,  'X', 
    '-',  'K',  'D',  'E',  '-',  'P',  'r',  'i', 
    'o',  'r',  'i',  't',  'y',  0x18,  0xc8,  0x6f, 
    'a',  'p',  'p',  'l',  'i',  'c',  'a',  't', 
    'i',  'o',  'n',  '/',  'z',  'i',  'p',  0xa7, 
    0x77,  'C',  'o',  'm',  'p',  'r',  'e',  's', 
    's',  'i',  'o',  'n',  'L',  'e',  'v',  'e', 
    'l',  'D',  'e',  'f',  'a',  'u',  'l',  't', 
    0x01,  0x73,  'C',  'o',  'm',  'p',  'r',  'e', 
    's',  's',  'i',  'o',  'n',  'L',  'e',  'v', 
    'e',  'l',  'M',  'a',  'x',  0x09,  0x73,  'C', 
    'o',  'm',  'p',  'r',  'e',  's',  's',  'i', 
    'o',  'n',  'L',  'e',  'v',  'e',  'l',  'M', 
    'i',  'n',  0x01,  0x78,  0x18,  'C',  'o',  'm', 
    'p',  'r',  'e',  's',  's',  'i',  'o',  'n', 
    'M',  'e',  't',  'h',  'o',  'd',  'D',  'e', 
    'f',  'a',  'u',  'l',  't',  0x67,  'D',  'e', 
    'f',  'l',  'a',  't',  'e',  0x72,  'C',  'o', 
    'm',  'p',  'r',  'e',  's',  's',  'i',  'o', 
    'n',  'M',  'e',  't',  'h',  'o',  'd',  's', 
    0xa2,  0x67,  'D',  'e',  'f',  'l',  'a',  't', 
    'e',  0x67,  'D',  'e',  'f',  'l',  'a',  't', 
    'e',  0x65,  'S',  't',  'o',  'r',  'e',  0x65, 
    'S',  't',  'o',  'r',  'e',  0x6f,  'S',  'u', 
    'p',  'p',  'o',  'r',  't',  's',  'T',  'e', 
    's',  't',  'i',  'n',  'g',  0xf4,  0x74,  'S', 
    'u',  'p',  'p',  'o',  'r',  't',  's',  'W', 
    'r',  'i',  't',  'e',  'C',  'o',  'm',  'm', 
    'e',  'n',  't',  0xf4, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(CliPzipPluginFactory, CliPzipPluginFactory, qt_pluginMetaDataV2_CliPzipPluginFactory)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_CliPzipPluginFactory[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x76,  'o',  'r',  'g',  '.',  'k',  'd', 
    'e',  '.',  'K',  'P',  'l',  'u',  'g',  'i', 
    'n',  'F',  'a',  'c',  't',  'o',  'r',  'y', 
    // "className"
    0x03,  0x74,  'C',  'l',  'i',  'P',  'z',  'i', 
    'p',  'P',  'l',  'u',  'g',  'i',  'n',  'F', 
    'a',  'c',  't',  'o',  'r',  'y', 
    // "MetaData"
    0x04,  0xa4,  0x67,  'K',  'P',  'l',  'u',  'g', 
    'i',  'n',  0xa8,  0x6b,  'D',  'e',  's',  'c', 
    'r',  'i',  'p',  't',  'i',  'o',  'n',  0x78, 
    0x34,  'H',  'i',  'g',  'h',  '-',  'p',  'e', 
    'r',  'f',  'o',  'r',  'm',  'a',  'n',  'c', 
    'e',  ' ',  'p',  'a',  'r',  'a',  'l',  'l', 
    'e',  'l',  ' ',  'Z',  'I',  'P',  ' ',  'c', 
    'o',  'm',  'p',  'r',  'e',  's',  's',  'i', 
    'o',  'n',  ' ',  'u',  's',  'i',  'n',  'g', 
    ' ',  'p',  'z',  'i',  'p',  0x72,  'D',  'e', 
    's',  'c',  'r',  'i',  'p',  't',  'i',  'o', 
    'n',  '[',  'z',  'h',  '_',  'C',  'N',  ']', 
    0x78,  0x2c,  uchar('\xe4'), uchar('\xbd'), uchar('\xbf'), uchar('\xe7'), uchar('\x94'), uchar('\xa8'),
    ' ',  'p',  'z',  'i',  'p',  ' ',  uchar('\xe8'), uchar('\xbf'),
    uchar('\x9b'), uchar('\xe8'), uchar('\xa1'), uchar('\x8c'), uchar('\xe9'), uchar('\xab'), uchar('\x98'), uchar('\xe6'),
    uchar('\x80'), uchar('\xa7'), uchar('\xe8'), uchar('\x83'), uchar('\xbd'), uchar('\xe5'), uchar('\xb9'), uchar('\xb6'),
    uchar('\xe8'), uchar('\xa1'), uchar('\x8c'), ' ',  'Z',  'I',  'P',  ' ', 
    uchar('\xe5'), uchar('\x8e'), uchar('\x8b'), uchar('\xe7'), uchar('\xbc'), uchar('\xa9'), 0x62,  'I', 
    'd',  0x71,  'k',  'e',  'r',  'f',  'u',  'f', 
    'f',  'l',  'e',  '_',  'c',  'l',  'i',  'p', 
    'z',  'i',  'p',  0x69,  'M',  'i',  'm',  'e', 
    'T',  'y',  'p',  'e',  's',  0x81,  0x6f,  'a', 
    'p',  'p',  'l',  'i',  'c',  'a',  't',  'i', 
    'o',  'n',  '/',  'z',  'i',  'p',  0x64,  'N', 
    'a',  'm',  'e',  0x6b,  'p',  'z',  'i',  'p', 
    ' ',  'p',  'l',  'u',  'g',  'i',  'n',  0x6b, 
    'N',  'a',  'm',  'e',  '[',  'z',  'h',  '_', 
    'C',  'N',  ']',  0x78,  0x1a,  'p',  'z',  'i', 
    'p',  ' ',  uchar('\xe9'), uchar('\xab'), uchar('\x98'), uchar('\xe6'), uchar('\x80'), uchar('\xa7'),
    uchar('\xe8'), uchar('\x83'), uchar('\xbd'), uchar('\xe5'), uchar('\x8e'), uchar('\x8b'), uchar('\xe7'), uchar('\xbc'),
    uchar('\xa9'), uchar('\xe6'), uchar('\x8f'), uchar('\x92'), uchar('\xe4'), uchar('\xbb'), uchar('\xb6'), 0x6c, 
    'S',  'e',  'r',  'v',  'i',  'c',  'e',  'T', 
    'y',  'p',  'e',  's',  0x81,  0x70,  'K',  'e', 
    'r',  'f',  'u',  'f',  'f',  'l',  'e',  '/', 
    'P',  'l',  'u',  'g',  'i',  'n',  0x67,  'V', 
    'e',  'r',  's',  'i',  'o',  'n',  0x63,  '1', 
    '.',  '0',  0x78,  0x19,  'X',  '-',  'K',  'D', 
    'E',  '-',  'K',  'e',  'r',  'f',  'u',  'f', 
    'f',  'l',  'e',  '-',  'R',  'e',  'a',  'd', 
    'W',  'r',  'i',  't',  'e',  0xf5,  0x6e,  'X', 
    '-',  'K',  'D',  'E',  '-',  'P',  'r',  'i', 
    'o',  'r',  'i',  't',  'y',  0x18,  0xc8,  0x6f, 
    'a',  'p',  'p',  'l',  'i',  'c',  'a',  't', 
    'i',  'o',  'n',  '/',  'z',  'i',  'p',  0xa7, 
    0x77,  'C',  'o',  'm',  'p',  'r',  'e',  's', 
    's',  'i',  'o',  'n',  'L',  'e',  'v',  'e', 
    'l',  'D',  'e',  'f',  'a',  'u',  'l',  't', 
    0x01,  0x73,  'C',  'o',  'm',  'p',  'r',  'e', 
    's',  's',  'i',  'o',  'n',  'L',  'e',  'v', 
    'e',  'l',  'M',  'a',  'x',  0x09,  0x73,  'C', 
    'o',  'm',  'p',  'r',  'e',  's',  's',  'i', 
    'o',  'n',  'L',  'e',  'v',  'e',  'l',  'M', 
    'i',  'n',  0x01,  0x78,  0x18,  'C',  'o',  'm', 
    'p',  'r',  'e',  's',  's',  'i',  'o',  'n', 
    'M',  'e',  't',  'h',  'o',  'd',  'D',  'e', 
    'f',  'a',  'u',  'l',  't',  0x67,  'D',  'e', 
    'f',  'l',  'a',  't',  'e',  0x72,  'C',  'o', 
    'm',  'p',  'r',  'e',  's',  's',  'i',  'o', 
    'n',  'M',  'e',  't',  'h',  'o',  'd',  's', 
    0xa2,  0x67,  'D',  'e',  'f',  'l',  'a',  't', 
    'e',  0x67,  'D',  'e',  'f',  'l',  'a',  't', 
    'e',  0x65,  'S',  't',  'o',  'r',  'e',  0x65, 
    'S',  't',  'o',  'r',  'e',  0x6f,  'S',  'u', 
    'p',  'p',  'o',  'r',  't',  's',  'T',  'e', 
    's',  't',  'i',  'n',  'g',  0xf4,  0x74,  'S', 
    'u',  'p',  'p',  'o',  'r',  't',  's',  'W', 
    'r',  'i',  't',  'e',  'C',  'o',  'm',  'm', 
    'e',  'n',  't',  0xf4, 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(CliPzipPluginFactory, CliPzipPluginFactory)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

namespace {
struct qt_meta_tag_ZN13CliPzipPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto CliPzipPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN13CliPzipPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CliPzipPlugin",
        "readStdout",
        "",
        "handleAll",
        "processFinished",
        "exitCode"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'readStdout'
        QtMocHelpers::SlotData<void(bool)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Slot 'readStdout'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'processFinished'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CliPzipPlugin, qt_meta_tag_ZN13CliPzipPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CliPzipPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<ReadWriteArchiveInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CliPzipPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CliPzipPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13CliPzipPluginE_t>.metaTypes,
    nullptr
} };

void CliPzipPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CliPzipPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->readStdout((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->readStdout(); break;
        case 2: _t->processFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *CliPzipPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CliPzipPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13CliPzipPluginE_t>.strings))
        return static_cast<void*>(this);
    return ReadWriteArchiveInterface::qt_metacast(_clname);
}

int CliPzipPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ReadWriteArchiveInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}
QT_WARNING_POP
