/****************************************************************************
** Meta object code from reading C++ file 'archivemanager.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../source/archivemanager/archivemanager.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'archivemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14ArchiveManagerE_t {};
} // unnamed namespace

template <> constexpr inline auto ArchiveManager::qt_create_metaobjectdata<qt_meta_tag_ZN14ArchiveManagerE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ArchiveManager",
        "signalError",
        "",
        "ErrorType",
        "eErrorType",
        "signalJobFinished",
        "ArchiveJob::JobType",
        "eJobType",
        "PluginFinishType",
        "eFinishType",
        "signalprogress",
        "dPercentage",
        "signalCurFileName",
        "strName",
        "signalFileWriteErrorName",
        "signalQuery",
        "Query*",
        "query",
        "signalCurArchiveName",
        "strArchiveName",
        "slotJobFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signalError'
        QtMocHelpers::SignalData<void(ErrorType)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'signalJobFinished'
        QtMocHelpers::SignalData<void(ArchiveJob::JobType, PluginFinishType, ErrorType)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 }, { 0x80000000 | 8, 9 }, { 0x80000000 | 3, 4 },
        }}),
        // Signal 'signalprogress'
        QtMocHelpers::SignalData<void(double)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 11 },
        }}),
        // Signal 'signalCurFileName'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'signalFileWriteErrorName'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'signalQuery'
        QtMocHelpers::SignalData<void(Query *)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'signalCurArchiveName'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Slot 'slotJobFinished'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ArchiveManager, qt_meta_tag_ZN14ArchiveManagerE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ArchiveManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ArchiveManagerE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ArchiveManagerE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14ArchiveManagerE_t>.metaTypes,
    nullptr
} };

void ArchiveManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ArchiveManager *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signalError((*reinterpret_cast<std::add_pointer_t<ErrorType>>(_a[1]))); break;
        case 1: _t->signalJobFinished((*reinterpret_cast<std::add_pointer_t<ArchiveJob::JobType>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<PluginFinishType>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<ErrorType>>(_a[3]))); break;
        case 2: _t->signalprogress((*reinterpret_cast<std::add_pointer_t<double>>(_a[1]))); break;
        case 3: _t->signalCurFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->signalFileWriteErrorName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->signalQuery((*reinterpret_cast<std::add_pointer_t<Query*>>(_a[1]))); break;
        case 6: _t->signalCurArchiveName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->slotJobFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< PluginFinishType >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< Query* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ArchiveManager::*)(ErrorType )>(_a, &ArchiveManager::signalError, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArchiveManager::*)(ArchiveJob::JobType , PluginFinishType , ErrorType )>(_a, &ArchiveManager::signalJobFinished, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArchiveManager::*)(double )>(_a, &ArchiveManager::signalprogress, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArchiveManager::*)(const QString & )>(_a, &ArchiveManager::signalCurFileName, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArchiveManager::*)(const QString & )>(_a, &ArchiveManager::signalFileWriteErrorName, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArchiveManager::*)(Query * )>(_a, &ArchiveManager::signalQuery, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ArchiveManager::*)(const QString & )>(_a, &ArchiveManager::signalCurArchiveName, 6))
            return;
    }
}

const QMetaObject *ArchiveManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ArchiveManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ArchiveManagerE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ArchiveManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ArchiveManager::signalError(ErrorType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ArchiveManager::signalJobFinished(ArchiveJob::JobType _t1, PluginFinishType _t2, ErrorType _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void ArchiveManager::signalprogress(double _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ArchiveManager::signalCurFileName(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ArchiveManager::signalFileWriteErrorName(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ArchiveManager::signalQuery(Query * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ArchiveManager::signalCurArchiveName(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
