/****************************************************************************
** Meta object code from reading C++ file 'dbusadpator.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../source/common/dbusadpator.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'dbusadpator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18ApplicationAdaptorE_t {};
} // unnamed namespace

template <> constexpr inline auto ApplicationAdaptor::qt_create_metaobjectdata<qt_meta_tag_ZN18ApplicationAdaptorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ApplicationAdaptor",
        "D-Bus Interface",
        "com.deepin.Compressor",
        "D-Bus Introspection",
        "  <interface name=\"com.deepin.Compressor\">\n    <method name=\"s"
        "howWindow\">\n      <arg direction=\"out\" type=\"b\"/>\n    </met"
        "hod>\n    <method name=\"hideWindow\">\n      <arg direction=\"out"
        "\" type=\"b\"/>\n    </method>\n    <method name=\"raiseWindow\">\n"
        "      <arg direction=\"out\" type=\"b\"/>\n    </method>\n    <met"
        "hod name=\"quitWindow\">\n      <arg direction=\"out\" type=\"b\"/"
        ">\n    </method>\n    <method name=\"compressFiles\">\n      <arg "
        "direction=\"in\" type=\"as\" name=\"filePaths\"/>\n      <arg dire"
        "ction=\"out\" type=\"b\"/>\n    </method>\n    <method name=\"extr"
        "actFiles\">\n      <arg direction=\"in\" type=\"s\" name=\"archive"
        "Path\"/>\n      <arg direction=\"in\" type=\"s\" name=\"destinatio"
        "nPath\"/>\n      <arg direction=\"out\" type=\"b\"/>\n    </method"
        ">\n    <method name=\"previewArchive\">\n      <arg direction=\"in"
        "\" type=\"s\" name=\"archivePath\"/>\n      <arg direction=\"out\""
        " type=\"b\"/>\n    </method>\n    <method name=\"raise\">\n      <"
        "arg direction=\"in\" type=\"s\" name=\"sFile\"/>\n      <arg direc"
        "tion=\"out\" type=\"b\"/>\n    </method>\n  </interface>\n",
        "raise",
        "",
        "sFile",
        "onActiveWindow",
        "pid",
        "showWindow",
        "hideWindow",
        "raiseWindow",
        "quitWindow",
        "compressFiles",
        "filePaths",
        "extractFiles",
        "archivePath",
        "destinationPath",
        "previewArchive",
        "focusChangedSlot",
        "QWidget*",
        "pwgt"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'raise'
        QtMocHelpers::SlotData<bool(const QString &)>(5, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'onActiveWindow'
        QtMocHelpers::SlotData<void(qint64)>(8, 6, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::LongLong, 9 },
        }}),
        // Slot 'showWindow'
        QtMocHelpers::SlotData<bool()>(10, 6, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'hideWindow'
        QtMocHelpers::SlotData<bool()>(11, 6, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'raiseWindow'
        QtMocHelpers::SlotData<bool()>(12, 6, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'quitWindow'
        QtMocHelpers::SlotData<bool()>(13, 6, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'compressFiles'
        QtMocHelpers::SlotData<bool(const QStringList &)>(14, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QStringList, 15 },
        }}),
        // Slot 'extractFiles'
        QtMocHelpers::SlotData<bool(const QString &, const QString &)>(16, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'previewArchive'
        QtMocHelpers::SlotData<bool(const QString &)>(19, 6, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'focusChangedSlot'
        QtMocHelpers::SlotData<void(QWidget *, QWidget *)>(20, 6, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 21, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
            {    3,    4 },
    });
    return QtMocHelpers::metaObjectData<ApplicationAdaptor, qt_meta_tag_ZN18ApplicationAdaptorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ApplicationAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.metaTypes,
    nullptr
} };

void ApplicationAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ApplicationAdaptor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->raise((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->onActiveWindow((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 2: { bool _r = _t->showWindow();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { bool _r = _t->hideWindow();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 4: { bool _r = _t->raiseWindow();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { bool _r = _t->quitWindow();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->compressFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->extractFiles((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->previewArchive((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->focusChangedSlot((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *ApplicationAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ApplicationAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18ApplicationAdaptorE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int ApplicationAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
